# This script is created by NSG2 beta1
# <http://wushoupong.googlepages.com/nsg>

#===================================
#        Initialization        
#===================================
#Create a ns simulator
set ns [new Simulator -multicast on]

#Open the NS trace file
set tracefile [open out.tr w]
$ns trace-all $tracefile

#Open the NAM trace file
set namfile [open out.nam w]
$ns namtrace-all $namfile

set f1 [open  sink0.tr w]
set r1 [open  s1.tr w]
set r2 [open  s2.tr w]
set j1 [open  jitter1.tr w]
set l1 [open  loss1.tr w]


#===================================
#     Simulation parameters setup
#===================================
set opt(stop)   100.0                         ;# time of simulation end
set opt(rate)	10kb
set opt(report_interval) 1000ms
$ns color 1 red
$ns color 30 purple
$ns color 31 bisque
$ns color 32 green



#===================================
#        Nodes Definition        
#===================================
#Create 7 nodes
set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]
set n4 [$ns node]
set n5 [$ns node]
set n6 [$ns node]
# Add some shapes
$n0 shape "box"
$n3 shape "box"
$n1 label "Multicast-Server"
$n2 label "TCP"
$n6 label "TCPSink"
$n0 label "IP-Router 1"
$n3 label "IP-Router 2"
$n4 label "Multicast-Receiver"
$n5 label "Multicast-Receiver"

#===================================
#        Links Definition        
#===================================
#Createlinks between nodes
$ns duplex-link $n0 $n3 1.0Mb 10ms DropTail
$ns queue-limit $n0 $n3 50
$ns duplex-link $n3 $n4 1.0Mb 10ms DropTail
$ns queue-limit $n3 $n4 50
$ns duplex-link $n3 $n5 1.0Mb 10ms DropTail
$ns queue-limit $n3 $n5 50
$ns duplex-link $n3 $n6 1.0Mb 10ms DropTail
$ns queue-limit $n3 $n6 50
$ns duplex-link $n1 $n0 2.0Mb 10ms DropTail
$ns queue-limit $n1 $n0 50
$ns duplex-link $n0 $n2 2.0Mb 10ms DropTail
$ns queue-limit $n0 $n2 50

#===================================
#        Agents Definition        
#===================================
#Setup a TCP connection
set tcp0 [new Agent/TCP]
$ns attach-agent $n2 $tcp0
set sink1 [new Agent/TCPSink]
$ns attach-agent $n6 $sink1
$ns connect $tcp0 $sink1
$tcp0 set packetSize_ 1064


#===================================
#        Applications Definition
#===================================
#Setup a FTP Application over TCP connection
set ftp0 [new Application/FTP]
$ftp0 attach-agent $tcp0
$ftp0 set rate_ 500kb/s
$ns at 1.0 "$ftp0 start"
$ns at 100.0 "$ftp0 stop"

#===================================
#        Smooth multicast congestion
#	 control over RTP
#===================================
# configure multicast protocol;
#set mproto CtrMcast
set mproto DM
# all nodes will contain multicast protocol agents;
set mrthandle [$ns mrtproto $mproto]
# allocate a multicast address;
set group [Node allocaddr]

set s0 [new Session/RTP ]
$s0 report-interval $opt(report_interval)
$s0 session_bw $opt(rate)
$s0 initial-rate  $opt(rate)
$s0 attach-node $n1
# enable congestion control
$s0 enable-control 1

set s1 [new Session/RTP ]
$s1 report-interval $opt(report_interval)
$s1 initial-rate  $opt(rate)
$s1 session_bw  $opt(rate)
$s1 attach-node $n4

set s2 [new Session/RTP ]
$s2 report-interval $opt(report_interval)
$s2 initial-rate  $opt(rate)
$s2 session_bw  $opt(rate)
$s2 attach-node $n5

$ns at 0.1 "$s0 join-group $group"
$ns at 0.1 "$s0 start"

$ns at 0.1 "$s1 join-group $group"
$ns at 0.1 "$s1 start"

$ns at 0.1 "$s2 join-group $group"
$ns at 0.1 "$s2 start"

# start multicast transmission
$ns at 1.0 "$s0 transmit $opt(rate)"



#===================================
#        Record functions
#===================================
$ns at 1.0 "record-sink1"
$ns at 1.0 "record-s1"
$ns at 1.0 "record-s2"
$ns at 1.0 "record-jitter1"
$ns at 1.0 "record-pktloss1"


# record the TCP throughput
proc record-sink1 {} {
        global sink1 f1
	#Get an instance of the simulator
	set ns [Simulator instance]
	#Set the time after which the procedure should be called again
        set time 0.5
	#How many bytes have been received by the traffic?
	set bw0 [$sink1 set bytes_]
        #Get the current time
        set now [$ns now]
	#Calculate the bandwidth (in MBit/s) and write it to the files
	puts $f1 "$now [expr $bw0/$time*8]"
        #Reset the bytes_ values on the traffic sinks
        $sink1 set bytes_ 0
       	#Re-schedule the procedure
        $ns at [expr $now+$time] "record-sink1"
}

# record s1 thrughput
proc record-s1 {} {
	global s1 r1
        set ns [Simulator instance]
        set time  0.5
	set bw  [$s1 set rx_recv_]
	set now [$ns now]
	puts $r1 "$now $bw"
        $ns at [expr $now+$time] "record-s1"
}

# record s2 thrughput
proc record-s2 {} {
        global s2 r2
        set ns [Simulator instance]
        set time  0.5
	set bw  [$s2 set rx_recv_]
	set now [$ns now]
        puts $r2 "$now $bw"
        $ns at [expr $now+$time] "record-s2"
}
#record jitter-s1
proc record-jitter1 {} {
	global s1 j1
        set ns [Simulator instance]
        set time  0.5
	set bw  [$s1 set jitter_]
	set now [$ns now]
	puts $j1 "$now $bw"
        $ns at [expr $now+$time] "record-jitter1"
}
#record packet loss ratio of s1 (% percent)
proc record-pktloss1 {} {
	global s1 l1
        set ns [Simulator instance]
        set time  0.5
	set bw  [$s1 set smooth_loss_]
	set now [$ns now]
	puts $l1 "$now $bw"
	$s1 set smooth_loss_ 0
        $ns at [expr $now+$time] "record-pktloss1"
}


#===================================
#        Termination        
#===================================
#Define a 'finish' procedure
proc finish {} {
     global ns f1 r1 r2 j1 l1 
     close $f1
     close $r1
     close $r2
     close $j1
     close $l1


     exec xgraph   sink0.tr s1.tr s2.tr  -geometry 800x400  &
     exec xgraph   jitter1.tr  -geometry 800x400  &
     exec xgraph   loss1.tr  -geometry 800x400  &



    global ns tracefile namfile
    $ns flush-trace
    close $tracefile
    close $namfile
    exec nam out.nam &
    exit 0
}

$ns at $opt(stop) "finish"
$ns run
